@@ -24,18 +24,26 @@
 #if __GNUC__ < 3 || (__GNUC__ == 3 && __GNUC_MINOR__ == 0)
 #error Xapian no longer supports GCC < 3.1
 #else
-#if !defined(__GXX_ABI_VERSION) || __GXX_ABI_VERSION != 102
+#if !defined(__GXX_ABI_VERSION) || __GXX_ABI_VERSION != 1002
 #error The C++ ABI version of compiler you are using does not match
 #error that of the compiler used to build the library. The versions
 #error must match or your program will not work correctly.
-#error The Xapian library was built with g++ 3.3.5
+#error The Xapian library was built with g++ 4.2.1
 #endif
 
+#ifdef _GLIBCXX_DEBUG
+#error You are compiling with _GLIBCXX_DEBUG defined, but the library
+#error was not compiled with this flag. The settings must match or your
+#error program will not work correctly.
 #endif
 #endif
+#endif
+
+/// The library was compiled with GCC's -fvisibility=hidden option.
+#define XAPIAN_ENABLE_VISIBILITY
 
 /// The version of Xapian as a C string literal.
-#define XAPIAN_VERSION "1.2.4"
+#define XAPIAN_VERSION "1.2.8"
 
 /** The major component of the Xapian version.
  * E.g. for Xapian 1.0.14 this would be: 1
@@ -50,7 +58,7 @@
 /** The revision component of the Xapian version.
  * E.g. for Xapian 1.0.14 this would be: 14
  */
-#define XAPIAN_REVISION 4
+#define XAPIAN_REVISION 8
 
 /// XAPIAN_HAS_BRASS_BACKEND Defined if the brass backend is enabled.
 #define XAPIAN_HAS_BRASS_BACKEND 1
